

// serial.cc
// serial port routines used by syncal calibration routines
// see below for IMPORTANT INFORMATION


#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>

#include "syncal.h"
#include "serial.h"


// NOTE: MAKE SURE TO SET THE PERMISSIONS ON TTYSx
// chmod a+rw /dev/ttySx
// DON'T FORGET!


extern int * in;		// declared in syncal.cc
int fd_serial;
struct termios oldtio, newtio;


void init_serial()
{
	printf("initializing serial port... ");
	fd_serial = open(MODEMDEVICE, O_RDWR | O_NOCTTY);
	if (fd_serial < 0) exit(-1); 

	tcgetattr(fd_serial,&oldtio);
	bzero(&newtio,sizeof(newtio));
	newtio.c_cflag = BAUDRATE | CLOCAL | CREAD | CS8;
	newtio.c_iflag = IGNPAR;
	newtio.c_oflag = 0;
	newtio.c_lflag = 0;
	newtio.c_cc[VMIN] = 0;
	newtio.c_cc[VTIME] = 0;
	tcsetattr(fd_serial,TCSAFLUSH,&newtio);
	printf("done\n");
}


void shutdown_serial()
{
	printf("shutting down serial port... ");
	tcsetattr(fd_serial,TCSAFLUSH,&oldtio);
	printf("done\n");
}


void send(int b)
{
	unsigned char a = (unsigned char) b;
	write(fd_serial,&a,1);
}



// serious kluging going on here.
// here's the issue:  we don't know how many bytes we're looking for
// so, we need to let a non-receive end the read.  so, we've put the
// serial port in the mode where min=0 and time=0 (see init_serial),
// in which it just does a read of whatever is there.  so not all of the
// data may have been passed by the time read decides to return.
// so, the fix is to return in you get nothing the first time, but if
// you do get something, then wait usleep(30000) and check again.  repeat
// until the stream has ended.
//
// the reason we need this kluge is that the mode where you use the
// time= field as a timeout value sucks when you don't know how many
// characters to receive.  in the mode min=0 and time=n, then the read
// returns after just one byte (no good).  in the mode min==x and time=n,
// the timeout doesn't happen until AFTER the first byte is received.  so,
// this mode is no good for polling because we want to return if no bytes
// have been received. 
int getserial()
{
	unsigned char a[MAX_BYTES];
	int nump;
	int total_pts = 0;
	do 
	{
		nump = read(fd_serial,&a[total_pts],MAX_BYTES);
		for (int i=total_pts; i < nump+total_pts; i++)
			in[i] = (int) a[i];
		if (nump) 
		{
			total_pts += nump;
			usleep(50000);
		}
  	} while(nump);
	return total_pts;
}


// end of serial.cc



