
// support.cc
// support functions used by syncal calibration program

#include <math.h>
#include "serial.h"


// send a command and arguments to the microcontroller.  the first byte is
// the command, the second byte is the number of args, and the other bytes
// are the args themselves
void send_cmd(int cmd, int numarg, int * first)
{
	send(cmd);
	send(numarg);
	if (! numarg) return;
	for (int i=0; i < numarg; i++) send(first[i]);
}


// convert the decimal value (LSD at first, ndig digits) into a double
double decimal_to_bin(int * first, int ndig)
{
	double result = 0.0;
	double temp;

	if (ndig != 1)
	{
		for (int i=0; i < ndig; i++)
		{
			temp = (double) (first[i]) * (double) (pow(10,i));
			result += temp;
		}
		result /= 10.0;
	}
	else result = (double) first[0] * 11.25;
	
	return result;
}


// convert the double passed to the function into decimal digits (LSD
// placed at first, ndig digits).
void bin_to_decimal(double val, int * first, int ndig)
{
	double run = val * 10.0;
	int temp;
	double sub;
	for (int i=0; i < ndig; i++)
	{
		temp = (int) (run / (double) (pow(10,(ndig-1)-i)));
		if (temp > 9)
		{
			for (int j=0; j < ndig; j++) first[j] = 0;
			return;
		}
		first[(ndig-1)-i] = temp;
		sub = (double) temp * (pow(10,(ndig-1)-i));
		run -= sub;
	}
}




// end of support.cc



